@@ -40,6 +40,12 @@
 #include "basicauth.h"
 #include "conf-tokens.h"
 
+#ifdef LINE_MAX
+#define TP_LINE_MAX LINE_MAX
+#else
+#define TP_LINE_MAX 1024
+#endif
+
 /*
  * The configuration directives are defined in the structure below.  Each
  * directive requires a regular expression to match against, and a
@@ -129,6 +135,7 @@ static HANDLE_FUNC (handle_filtercasesensitive);
 static HANDLE_FUNC (handle_filterdefaultdeny);
 static HANDLE_FUNC (handle_filterextended);
 static HANDLE_FUNC (handle_filterurls);
+static HANDLE_FUNC (handle_filtertype);
 #endif
 static HANDLE_FUNC (handle_group);
 static HANDLE_FUNC (handle_listen);
@@ -228,6 +235,7 @@ struct {
         STDCONF (filterextended, BOOL, handle_filterextended),
         STDCONF (filterdefaultdeny, BOOL, handle_filterdefaultdeny),
         STDCONF (filtercasesensitive, BOOL, handle_filtercasesensitive),
+        STDCONF (filtertype, "(bre|ere|fnmatch)", handle_filtertype),
 #endif
 #ifdef REVERSE_SUPPORT
         /* Reverse proxy arguments */
@@ -409,7 +417,7 @@ static int check_match (struct config_s *conf, const char *line,
  */
 static int config_parse (struct config_s *conf, FILE * f)
 {
-        char buffer[LINE_MAX], *p, *q, c;
+        char buffer[TP_LINE_MAX], *p, *q, c;
         const struct config_directive_entry *e;
         unsigned long lineno = 1;
 
@@ -505,7 +513,7 @@ int reload_config_file (const char *config_fname, struct config_s *conf)
                 goto done;
         }
 
-        if (!conf->user) {
+        if (!conf->user && !geteuid()) {
                 log_message (LOG_WARNING, "You SHOULD set a UserName in the "
                              "config file. Using current user instead.");
         }
@@ -701,6 +709,8 @@ static HANDLE_FUNC (handle_xtinyproxy)
 #ifdef XTINYPROXY_ENABLE
         return set_bool_arg (&conf->add_xtinyproxy, line, &match[2]);
 #else
+        if(!get_bool_arg(line, &match[2]))
+                return 0;
         fprintf (stderr,
                  "XTinyproxy NOT Enabled! Recompile with --enable-xtinyproxy\n");
         return 1;
@@ -944,6 +954,11 @@ static HANDLE_FUNC (handle_basicauth)
 }
 
 #ifdef FILTER_ENABLE
+
+static void warn_deprecated(const char *arg, unsigned long lineno) {
+        CP_WARN ("deprecated option %s", arg);
+}
+
 static HANDLE_FUNC (handle_filter)
 {
         return set_string_arg (&conf->filter, line, &match[2]);
@@ -951,26 +966,53 @@ static HANDLE_FUNC (handle_filter)
 
 static HANDLE_FUNC (handle_filterurls)
 {
-        return set_bool_arg (&conf->filter_url, line, &match[2]);
+        conf->filter_opts |=
+             get_bool_arg (line, &match[2]) * FILTER_OPT_URL;
+        return 0;
 }
 
 static HANDLE_FUNC (handle_filterextended)
 {
-        return set_bool_arg (&conf->filter_extended, line, &match[2]);
+        warn_deprecated("FilterExtended, use FilterType", lineno);
+        conf->filter_opts |=
+             get_bool_arg (line, &match[2]) * FILTER_OPT_TYPE_ERE;
+        return 0;
 }
 
 static HANDLE_FUNC (handle_filterdefaultdeny)
 {
         assert (match[2].rm_so != -1);
-
-        if (get_bool_arg (line, &match[2]))
-                filter_set_default_policy (FILTER_DEFAULT_DENY);
+        conf->filter_opts |=
+            get_bool_arg (line, &match[2]) * FILTER_OPT_DEFAULT_DENY;
         return 0;
 }
 
 static HANDLE_FUNC (handle_filtercasesensitive)
 {
-        return set_bool_arg (&conf->filter_casesensitive, line, &match[2]);
+        conf->filter_opts |=
+            get_bool_arg (line, &match[2]) * FILTER_OPT_CASESENSITIVE;
+        return 0;
+}
+
+static HANDLE_FUNC (handle_filtertype)
+{
+        static const struct { unsigned short flag; char type[8]; }
+        ftmap[] = {
+             {FILTER_OPT_TYPE_ERE,	"ere"},
+             {FILTER_OPT_TYPE_BRE,	"bre"},
+             {FILTER_OPT_TYPE_FNMATCH,	"fnmatch"},
+        };
+        char *type;
+        unsigned i;
+        type = get_string_arg(line, &match[2]);
+        if (!type) return -1;
+
+        for(i=0;i<sizeof(ftmap)/sizeof(ftmap[0]);++i)
+                if(!strcmp(ftmap[i].type, type))
+                        conf->filter_opts |= ftmap[i].flag;
+
+        safefree (type);
+        return 0;
 }
 #endif
 
