@@ -268,28 +268,34 @@ establish_http_connection (struct conn_s *connptr, struct request_s *request)
                 /* host is an IPv6 address literal, so surround it with
                  * [] */
                 return write_message (connptr->server_fd,
-                                      "%s %s HTTP/1.0\r\n"
+                                      "%s %s HTTP/1.%u\r\n"
                                       "Host: [%s]%s\r\n"
                                       "Connection: close\r\n",
                                       request->method, request->path,
+                                      connptr->protocol.major != 1 ? 0 :
+                                               connptr->protocol.minor,
                                       request->host, portbuff);
         } else if (connptr->upstream_proxy &&
                    connptr->upstream_proxy->type == PT_HTTP &&
                    connptr->upstream_proxy->ua.authstr) {
                 return write_message (connptr->server_fd,
-                                      "%s %s HTTP/1.0\r\n"
+                                      "%s %s HTTP/1.%u\r\n"
                                       "Host: %s%s\r\n"
                                       "Connection: close\r\n"
                                       "Proxy-Authorization: Basic %s\r\n",
                                       request->method, request->path,
+                                      connptr->protocol.major != 1 ? 0 :
+                                               connptr->protocol.minor,
                                       request->host, portbuff,
                                       connptr->upstream_proxy->ua.authstr);
         } else {
                 return write_message (connptr->server_fd,
-                                      "%s %s HTTP/1.0\r\n"
+                                      "%s %s HTTP/1.%u\r\n"
                                       "Host: %s%s\r\n"
                                       "Connection: close\r\n",
                                       request->method, request->path,
+                                      connptr->protocol.major != 1 ? 0 :
+                                               connptr->protocol.minor,
                                       request->host, portbuff);
         }
 }
@@ -383,10 +389,17 @@ BAD_REQUEST_ERROR:
                  * we'll be closing anyway.
                  */
                 char *reverse_url;
+                int reverse_status;
 
-                reverse_url = reverse_rewrite_url (connptr, hashofheaders, url);
+                reverse_url = reverse_rewrite_url (connptr, hashofheaders, url, &reverse_status);
 
                 if (reverse_url != NULL) {
+                        if (reverse_status == 301) {
+                                char buf[PATH_MAX];
+                                snprintf (buf, sizeof buf, "Location: %s\r\n", reverse_url);
+                                send_http_headers (connptr, 301, "Moved Permanently", buf);
+                                goto fail;
+                        }
                         safefree (url);
                         url = reverse_url;
                 } else if (config->reverseonly) {
@@ -458,22 +471,16 @@ BAD_REQUEST_ERROR:
          * Filter restricted domains/urls
          */
         if (config->filter) {
-                if (config->filter_url)
-                        ret = filter_run (url);
-                else
-                        ret = filter_run (request->host);
+                int fu = config->filter_opts & FILTER_OPT_URL;
+                ret = filter_run (fu ? url : request->host);
 
                 if (ret) {
                         update_stats (STAT_DENIED);
 
-                        if (config->filter_url)
-                                log_message (LOG_NOTICE,
-                                             "Proxying refused on filtered url \"%s\"",
-                                             url);
-                        else
-                                log_message (LOG_NOTICE,
-                                             "Proxying refused on filtered domain \"%s\"",
-                                             request->host);
+                        log_message (LOG_NOTICE,
+                                     "Proxying refused on filtered %s \"%s\"",
+                                     fu ? "url" : "domain",
+                                     fu ? url : request->host);
 
                         indicate_http_error (connptr, 403, "Filtered",
                                              "detail",
@@ -510,7 +517,7 @@ fail:
  * server headers can be processed.
  *	- rjkaes
  */
-static int pull_client_data (struct conn_s *connptr, long int length)
+static int pull_client_data (struct conn_s *connptr, long int length, int iehack)
 {
         char *buffer;
         ssize_t len;
@@ -535,39 +542,75 @@ static int pull_client_data (struct conn_s *connptr, long int length)
                 length -= len;
         } while (length > 0);
 
-        /*
-         * BUG FIX: Internet Explorer will leave two bytes (carriage
-         * return and line feed) at the end of a POST message.  These
-         * need to be eaten for tinyproxy to work correctly.
-         */
-        ret = socket_nonblocking (connptr->client_fd);
-        if (ret != 0) {
-                log_message(LOG_ERR, "Failed to set the client socket "
-                            "to non-blocking: %s", strerror(errno));
-                goto ERROR_EXIT;
-        }
-
-        len = recv (connptr->client_fd, buffer, 2, MSG_PEEK);
-
-        ret = socket_blocking (connptr->client_fd);
-        if (ret != 0) {
-                log_message(LOG_ERR, "Failed to set the client socket "
-                            "to blocking: %s", strerror(errno));
-                goto ERROR_EXIT;
-        }
-
-        if (len < 0 && errno != EAGAIN)
-                goto ERROR_EXIT;
-
-        if ((len == 2) && CHECK_CRLF (buffer, len)) {
-                ssize_t bytes_read;
-
-                bytes_read = read (connptr->client_fd, buffer, 2);
-                if (bytes_read == -1) {
-                        log_message
-                                (LOG_WARNING,
-                                 "Could not read two bytes from POST message");
+        if (iehack) {
+                /*
+                 * BUG FIX: Internet Explorer will leave two bytes (carriage
+                 * return and line feed) at the end of a POST message.  These
+                 * need to be eaten for tinyproxy to work correctly.
+                 */
+                ret = socket_nonblocking (connptr->client_fd);
+                if (ret != 0) {
+                        log_message(LOG_ERR, "Failed to set the client socket "
+                                    "to non-blocking: %s", strerror(errno));
+                        goto ERROR_EXIT;
                 }
+        
+                len = recv (connptr->client_fd, buffer, 2, MSG_PEEK);
+        
+                ret = socket_blocking (connptr->client_fd);
+                if (ret != 0) {
+                        log_message(LOG_ERR, "Failed to set the client socket "
+                                    "to blocking: %s", strerror(errno));
+                        goto ERROR_EXIT;
+                }
+        
+                if (len < 0 && errno != EAGAIN)
+                        goto ERROR_EXIT;
+        
+                if ((len == 2) && CHECK_CRLF (buffer, len)) {
+                        ssize_t bytes_read;
+        
+                        bytes_read = read (connptr->client_fd, buffer, 2);
+                        if (bytes_read == -1) {
+                                log_message
+                                        (LOG_WARNING,
+                                         "Could not read two bytes from POST message");
+                        }
+                }
+        }
+
+        safefree (buffer);
+        return 0;
+
+ERROR_EXIT:
+        safefree (buffer);
+        return -1;
+}
+
+/* pull chunked client data */
+static int pull_client_data_chunked (struct conn_s *connptr) {
+        char *buffer = 0;
+        ssize_t len;
+        long chunklen;
+
+        while(1) {
+                if (buffer) safefree(buffer);
+                len = readline (connptr->client_fd, &buffer);
+
+                if (len <= 0)
+                        goto ERROR_EXIT;
+
+                if (!connptr->error_variables) {
+                        if (safe_write (connptr->server_fd, buffer, len) < 0)
+                                goto ERROR_EXIT;
+                }
+
+                chunklen = strtol (buffer, (char**)0, 16);
+
+                if (pull_client_data (connptr, chunklen+2, 0) < 0)
+                        goto ERROR_EXIT;
+
+                if(!chunklen) break;
         }
 
         safefree (buffer);
@@ -774,7 +817,7 @@ static int remove_connection_headers (orderedmap hashofheaders)
 
 /*
  * If there is a Content-Length header, then return the value; otherwise, return
- * a negative number.
+ * -1.
  */
 static long get_content_length (orderedmap hashofheaders)
 {
@@ -789,6 +832,13 @@ static long get_content_length (orderedmap hashofheaders)
         return content_length;
 }
 
+static int is_chunked_transfer (orderedmap hashofheaders)
+{
+        char *data;
+        data = orderedmap_find (hashofheaders, "transfer-encoding");
+        return data ? !strcmp (data, "chunked") : 0;
+}
+
 /*
  * Search for Via header in a hash of headers and either write a new Via
  * header, or append our information to the end of an existing Via header.
@@ -883,6 +933,10 @@ process_client_headers (struct conn_s *connptr, orderedmap hashofheaders)
          */
         connptr->content_length.client = get_content_length (hashofheaders);
 
+        /* Check whether client sends chunked data. */
+        if (connptr->content_length.client == -1 && is_chunked_transfer (hashofheaders))
+                connptr->content_length.client = -2;
+
         /*
          * See if there is a "Connection" header.  If so, we need to do a bit
          * of processing. :)
@@ -947,8 +1001,9 @@ process_client_headers (struct conn_s *connptr, orderedmap hashofheaders)
 PULL_CLIENT_DATA:
         if (connptr->content_length.client > 0) {
                 ret = pull_client_data (connptr,
-                                        connptr->content_length.client);
-        }
+                                        connptr->content_length.client, 1);
+        } else if (connptr->content_length.client == -2)
+                ret = pull_client_data_chunked (connptr);
 
         return ret;
 }
@@ -1231,7 +1286,6 @@ connect_to_upstream_proxy(struct conn_s *connptr, struct request_s *request)
 	unsigned short port;
 	size_t ulen, passlen;
 
-	struct hostent *host;
 	struct upstream *cur_upstream = connptr->upstream_proxy;
 
 	ulen = cur_upstream->ua.user ? strlen(cur_upstream->ua.user) : 0;
@@ -1248,10 +1302,13 @@ connect_to_upstream_proxy(struct conn_s *connptr, struct request_s *request)
 		buff[1] = 1; /* connect command */
 		port = htons(request->port);
 		memcpy(&buff[2], &port, 2); /* dest port */
-		host = gethostbyname(request->host);
-		memcpy(&buff[4], host->h_addr_list[0], 4); /* dest ip */
-		buff[8] = 0; /* user */
-		if (9 != safe_write(connptr->server_fd, buff, 9))
+		memcpy(&buff[4], "\0\0\0\1" /* socks4a fake ip */
+				 "\0" /* user */, 5);
+		len = strlen(request->host);
+		if(len>255)
+			return -1;
+		memcpy(&buff[9], request->host, len+1);
+		if (9+len+1 != safe_write(connptr->server_fd, buff, 9+len+1))
 			return -1;
 		if (8 != safe_read(connptr->server_fd, buff, 8))
 			return -1;
@@ -1361,7 +1418,7 @@ connect_to_upstream (struct conn_s *connptr, struct request_s *request)
                 log_message (LOG_WARNING,
                              "No upstream proxy defined for %s.",
                              request->host);
-                indicate_http_error (connptr, 404,
+                indicate_http_error (connptr, 502,
                                      "Unable to connect to upstream proxy.");
                 return -1;
         }
@@ -1373,7 +1430,7 @@ connect_to_upstream (struct conn_s *connptr, struct request_s *request)
         if (connptr->server_fd < 0) {
                 log_message (LOG_WARNING,
                              "Could not connect to upstream proxy.");
-                indicate_http_error (connptr, 404,
+                indicate_http_error (connptr, 502,
                                      "Unable to connect to upstream proxy",
                                      "detail",
                                      "A network error occurred while trying to "
@@ -1449,8 +1506,8 @@ get_request_entity(struct conn_s *connptr)
                 nread = read_buffer (connptr->client_fd, connptr->cbuffer);
                 if (nread < 0) {
                         log_message (LOG_ERR,
-                                     "Error reading readable client_fd %d",
-                                     connptr->client_fd);
+                                     "Error reading readable client_fd %d (%s)",
+                                     connptr->client_fd, strerror(errno));
                         ret = -1;
                 } else {
                         log_message (LOG_INFO,
@@ -1502,6 +1559,9 @@ static void handle_connection_failure(struct conn_s *connptr, int got_headers)
  * when we start the relay portion. This makes most of the original
  * tinyproxy code, which was confusing, redundant. Hail progress.
  * 	- rjkaes
+
+ * this function is called directly from child_thread() with the newly
+ * received fd from accept(). 
  */
 void handle_connection (struct conn_s *connptr, union sockaddr_union* addr)
 {
@@ -1513,7 +1573,6 @@ void handle_connection (struct conn_s *connptr, union sockaddr_union* addr)
         int got_headers = 0, fd = connptr->client_fd;
         size_t i;
         struct request_s *request = NULL;
-        struct timeval tv;
         orderedmap hashofheaders = NULL;
 
         char sock_ipaddr[IP_LENGTH];
@@ -1535,12 +1594,7 @@ void handle_connection (struct conn_s *connptr, union sockaddr_union* addr)
                 return;
         }
 
-        tv.tv_usec = 0;
-        tv.tv_sec = config->idletimeout;
-        setsockopt(fd, SOL_SOCKET, SO_SNDTIMEO, (void*) &tv, sizeof(tv));
-        tv.tv_usec = 0;
-        tv.tv_sec = config->idletimeout;
-        setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, (void*) &tv, sizeof(tv));
+        set_socket_timeout(fd);
 
         if (connection_loops (addr))  {
                 log_message (LOG_CONN,
@@ -1568,11 +1622,7 @@ void handle_connection (struct conn_s *connptr, union sockaddr_union* addr)
 
         if (read_request_line (connptr) < 0) {
                 update_stats (STAT_BADCONN);
-                indicate_http_error (connptr, 408, "Timeout",
-                                     "detail",
-                                     "Server timeout waiting for the HTTP request "
-                                     "from the client.", NULL);
-                HC_FAIL();
+                goto done;
         }
 
         /*
