@@ -23,6 +23,8 @@
 #define _TINYPROXY_REQS_H_
 
 #include "common.h"
+#include "sock.h"
+#include "conns.h"
 
 /*
  * Port constants for HTTP (80) and SSL (443)
@@ -43,6 +45,6 @@ struct request_s {
         char *path;
 };
 
-extern void handle_connection (int fd);
+extern void handle_connection (struct conn_s *, union sockaddr_union* addr);
 
 #endif
